function [sig,trend,fs,BL]=testTrend_MT(data)
% 
% Trend Test based on a Monte Carlo method (Circular Block Bootstrap; 
% see [1] and [2])
% 
% Input:
% 	data    : data series (Ntime,Nstations)
% 	
% Output:    
% 	sig     : 1xNstations matrix with trend significance for each stations
%               (1=significant trend; 0=no significant trend)
% 	trend   : 1xNstations matrix with trend values.
%       fs      : Field significance
%       BL      : Universal* Block Lenght (* see [1])
% 
% Needs:
%   1) opt_block_length_REV_dec07
%   (http://econ.duke.edu/~ap172/opt_block_length_REV_dec07.txt, [3])
%   2) mlag
%   (http://econ.duke.edu/~ap172/mlag.m.txt, [3])
%   3) segmcirc from the bootstrap_toolbox
%   (http://www.csp.curtin.edu.au/downloads/bootstrap_toolbox.html, [4])
% 
% Limitation: 
%   1) this code does not work with NaN
%   2) it may take several minutes
%
% Example:
% [sig,trend,fs,BL]=testTrend_MT(data)
%
% References:  
% [1] Kiktev, D., D. M. H. Sexton, L. Alexander, and C. K. Folland, 2003: 
%   Comparison of modeled and observed trends in indices of daily climate 
%   extremes. J. Climate, 16, 3560–3571.
% [2] Turco, M., Llasat, M.C., 2011: Trends in indices of daily 
%    precipitation extremes in Catalonia (NE Spain), 1951-2003. Accepted 
%    in NHESS. 
% [3] Politis, D. N., and White, H.: Automatic Block-Length Selection for 
%   the Dependent Bootstrap, Economet Rev 23, 53-70, 2004
% [4] Zoubir A. M., Boashash B. "The Bootstrap and Its Application in
%   signal Processing". IEEE Signal Proc Mag 15, 56-76, 1998
%
% Code owner: Marco Turco  (07-Jan-2010) 
% Contact: marco_turco AT yahoo DOT it 

%% fixed parameter

NB=1000; % bootstrap replications
alpha=5; % significance level at 5%

%% Variable declaration

[ndata,Nest]=size(data);
sig=NaN*zeros(1,Nest);
trend=NaN*zeros(1,Nest);

res=NaN*zeros(ndata,Nest); %residuals
BLS=NaN*zeros(2,Nest); % Block Lenght for each station
jcat=NaN*zeros(1,4); % j index to cat the block together
newdato=NaN*zeros(ndata,NB);

res_B=NaN*zeros(ndata,NB); % bostrapped residuals
pboot=NaN*zeros(1,NB); %1000 new plausible trend values 
dtrend=NaN*zeros(ndata,NB); %1000 new plausible trend values - actual trend
sigarea=NaN*zeros(1,NB); %significant area
ci=NaN*zeros(Nest,2);

%% Block Lenght selection
%STEP 1: http://econ.duke.edu/~ap172/opt_block_length_REV_dec07.txt
BLS=opt_block_length_REV_dec07(data);
BLS(2,:)=sort(BLS(2,:));
%STEP 2: universal BL-value 
BL=round(BLS(2,round(0.9*Nest))); 
if (BL<1) 
   BL=1;
end
%Are the data divisible for the block lenght?
if (mod(ndata,BL)==0)
        k=ndata/BL;
else 
        k=ceil(ndata/BL);
end

rescirc=NaN*zeros(BL,k);   %wrapped data series
res_tmp=NaN*zeros((k*BL),NB); %temporaly bostrapped residuals

%% Spatial interdependence of the station data
% --> same resampling sequence for each station within each iteration of 
%     the bootstrap
ind=bootrsp(1:k,NB);

%% Main

for i=1:Nest
    %% Linear trend
    dato=data(:,i);
	p=polyfit([1:length(dato)]',dato,1);
    trend(i)=p(1);

    %% Residual
    res=dato-polyval(p,[1:length(dato)]'); 
    
    %% Resample residual time series 
    %Construction of the wrapped data series {X1,...,XN+b-1} where XN+k=Xk
    %http://www.csp.curtin.edu.au/downloads/bootstrap_toolbox.html
    rescirc=segmcirc(res,BL,1,k); 
   
    for ib=1:NB
       %randomly select the number of block and join them together  
       for icat=1:k
          jcat=(((icat*BL)-(BL-1)):icat*BL);
          res_tmp(jcat,ib)=vertcat(rescirc(:,ind(icat,ib)) );
       end
       if (mod(length(dato),BL)~=0)
         %the data are not divisible for the block length
         res_B(:,ib)=res_tmp(1:ndata,ib);
       else  
         res_B(:,ib)=res_tmp(:,ib);  
       end
       %Add original linear trend onto NB resampled time series of noise
       newdato(:,ib)=(p(2)+p(1)*[1:length(dato)])'+res_B(:,ib);
       %re-estimate station trends to produce NB new plausible trend values 
       p2=polyfit([1:length(newdato(:,ib))]',newdato(:,ib),1);
       pboot(ib)=p2(1);
       %difference between each of the NB patterns of pseudo-trends and the 
       % actual trend pattern. (field significance of trends)
       dtrend(i,ib)=pboot(ib)-trend(i);
    end
    % trend significance for each of the stations
    if (prctile(pboot,(100-(alpha/2)))<=0) %significant negative trend   
       sig(i)=1;
    elseif (prctile(pboot,(alpha/2))>=0) %significant positive trend
       sig(i)=1;
    elseif (prctile(pboot,(alpha/2))<=0) && (prctile(pboot,(100-(alpha/2)))>=0) 
       sig(i)=0;
    end   
end

%% field significance of trends
% step 1:
% dtrend(ndata,NB) fields was used to estimate the 95% confidence interval 
% for zero trend at each grid point  
for i=1:Nest
   ci(i,1)=prctile(dtrend(i,:),(alpha/2));
   ci(i,2)=prctile(dtrend(i,:),(100-(alpha/2)));
end    
   
% step 2:
% Using these confidence intervals calculate the number of points with a 
% significant trend due to chance, 
for ib=1:NB
    sigarea(ib)=length(find(dtrend(:,ib)<=ci(:,1)))+length(find(dtrend(:,ib)>=ci(:,2)));
end  

sidaror=length(find(trend(:)<=ci(:,1)))+length(find(trend(:)>=ci(:,2)));
fs=100*(length(find(sidaror>=sigarea))/NB);
  